;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x54883c62e07532c30d7b65503eeb9099 . "./code.cg2js/cg-status-bar.fasl")
(#x24636221630357eee81c6b5a7e41be10 . "./code.cg2js/cg-hotspot.fasl")
(#x53cf228945a087037da3a75de610961f . "./code.cg2js/cg-toolbar.fasl")
(#xe7e5c133baf8de7ded09fc4dba94ad19 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x5f0055543d07778de353ac8deb4965ab . "./code.cg2js/cg-tooltip.fasl")
(#xf8e4091626f3481e2ef07e0ce6a7f4ed . "./code.cg2js/cg-calendar.fasl")
(#x4410b66b2fe04786af95d921f530384a . "./code.cg2js/cg-item-list.fasl")
(#xdf53021546c114fdd2cdc1850c19c59e . "./code.cg2js/cg-bitmap-stream.fasl")
(#x67d1213d8eaf2f2400d82c03fb22f499 . "./code.cg2js/cg-trackbar.fasl")
(#xdbec2dc7a90d62e72500a2810447f7e4 . "./code.cg2js/cg-class-grid.fasl")
(#xd00f14daade89656eb9bba9f86beba77 . "./code.cg2js/cg-dialog-item.fasl")
(#x9630914f8eed9520f65f7476aee80212 . "./code.cg2js/cg-comtab.fasl")
(#x04cd3aa392106190d2902d63efe9e44d . "./code.cg2js/cg-lisp-text.fasl")
(#x36197c808e500693bd7e80b9bbd85eca . "./code.cg2js/cg-button.fasl")
(#x995b6971d84f773c0f69badbd6a9a64d . "./code.cg2js/cg-printing.fasl")
(#xbe525784885409219fbbd601628b387b . "./code.cg2js/cg-pixmap-printing.fasl")
(#xd17a6558c6131cea92ba5a4924b3ec12 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x35b085dbf60ccfd2edec04096497e9bc . "./code.cg2js/cg-pixmap-widget.fasl")
(#xd728c5935e5b39e828d72263368fccd6 . "./code.cg2js/cg-selected-object.fasl")
(#xd106b044de4054c62582a1d893424b2a . "./code.cg2js/cg-text-or-combo.fasl")
(#xe3e7713348c7cac7eb0df6129bc8142b . "./code.cg2js/cg-graphics-context.fasl")
(#x46a49b0a38ac5f4c64e896d251c20b81 . "./code.cg2js/cg-edit-in-place.fasl")
(#x3061b803fd8f2a71ce196f9f7a3f5be8 . "./code.cg2js/cg-string-dialog.fasl")
(#x6dc7c2f5ddcbc47a2c413760a38c05a8 . "./code.cg2js/cg-list-view.fasl")
(#xf9106b0b6de54634786af5ecd1e07754 . "./code.cg2js/cg-intersect.fasl")
(#xea939ccd215bebef936a9bb73e07399b . "./code.cg2js/cg-text-widget.fasl")
(#xcb6618d64d05d833614f430bf77ea287 . "./code.cg2js/cg-directory-dialog.fasl")
(#x2be2ca2026d3644476963bdbd397d3e9 . "./code.cg2js/cg-file-dialog.fasl")
(#x1fe516385542357f16a0ba106186e05f . "./code.cg2js/cg-dde-utils.fasl")
(#x4631b41b2128b643ca9396061e6d1d77 . "./code.cg2js/cg-nodes.fasl")
(#xa62ed2112e72d4e649bce8301757cc3f . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xf6c854facf98f37fc66d86b3363ca93d . "./code.cg2js/cg-up-down-control.fasl")
(#x2a02600aa5680cf852a7c29705a94385 . "./code.cg2js/cg-get-position.fasl")
(#x078c9891f69dfd1113e6780d5797fd4e . "./code.cg2js/cg-toggling-widget.fasl")
(#xd3cc6facaa6c95e6331a7a4e04018125 . "./code.cg2js/cg-acache.fasl")
(#xa67189483106ed473b107d411ac789f1 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x7ca5afcee1eaecbb4ac5bc1e140285d9 . "./code.cg2js/cg-base.fasl")
(#x865756d27a9638cf3809c86cd86bd7af . "./code.cg2js/cg-drag-and-drop.fasl")
(#x8d22ecf7fdfd43143545788c649b9ba8 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xaf2c2be41477a7d9a9ace201420baeb2 . "./code.cg2js/cg-property.fasl")
(#x87b436874a266b44e361770844ee45d5 . "./code.cg2js/cg-editable-text.fasl")
(#x806e9378a017a92b4df2ac82709302e2 . "./code.cg2js/cg-lamp.fasl")
(#x11a0904bca031e429cde4b542db5f6db . "./code.cg2js/cg-progress-indicator.fasl")
(#xc22a1ac4a77db5981d9932f277fa6ee3 . "./code.cg2js/cg-chart-widget.fasl")
(#xc818763c4cf511beb6ab848db51b3419 . "./code.cg2js/cg-common-control.fasl")
(#xfafe56eab8b8a0c3d8f7a3ea0e2eab22 . "./code.cg2js/cg-font-dialog.fasl")
(#x34a4755a2b9208f207fd01c339b5932b . "./code.cg2js/cg-gesture-emulation.fasl")
(#x4d1bd709a5966056a9ebd1cfe5c71ccb . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x77d94e8c69ea7e760c9a72c4ce4804ce . "./code.cg2js/cg-os-window.fasl")
(#x31f6a8965a162793a69b11f6fc9cc729 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x5822b080bea9623de3aa8713f789f0a2 . "./code.cg2js/cg-pixmap.fasl")
(#xf16bf504bf950494093e8c9e006d86f5 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x0aac782e1c472386d0068d53e3b38468 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x6aab0f3390dad85e131d0ebcc68b7cde . "./code.cg2js/cg-scaling-stream.fasl")
(#xf738561eded64d899e48484442f089fd . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xef5f3921ed14ed00759c78889fea4058 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x0b62045eea7cee3b6b6b16c2d621b6c8 . "./code.cg2js/cg-timer.fasl")
(#xf7a07e15e3f36f2e6379cfdd90bdee4c . "./code.cg2js/cg-scroll-bar.fasl")
(#x27460dd3c399419ee0b42d4c054cebb5 . "./code.cg2js/cg-caret.fasl")
(#xbae2d21efbff9d468bc6d5133d7196b3 . "./code.cg2js/cg-template-string.fasl")
(#xef8a59790393f7a3345775ea10c30bdd . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x74ac56db5ea5e750afd2964f593be3e9 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x3b3936702a6325bb2a77a981f6b346d5 . "./code.cg2js/cg-combo-box.fasl")
(#x91c057e481bbb9970c63a201b6ff697a . "./code.cg2js/cg-text-edit-pane.fasl")
(#x315b91384b10ed30f79496255eac88d6 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x713430f47743b253cce9be963556c671 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x456bd521c48bf213fa7badc47fd85872 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x5a30e3211f04c15c3ab01e6548ba1e90 . "./code.cg2js/cg-player.fasl")
(#x69cd33a60e2a6c9875e60b645e6ebb78 . "./code.cg2js/cg-outline.fasl")
(#xccc74712d8f55f6d8806a709e0a40de5 . "./code.cg2js/cg-class-support.fasl")
(#x1210cefb0fc14a210bc8ee9672af8993 . "./code.cg2js/cg-lettered-menu.fasl")
(#x388c6c7bb7b9d7fd820741da331194a6 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x12fd5480bbd0029f00bf8cc29da00cba . "./code.cg2js/cg-fill-texture.fasl")
(#xbda42c4e6d30681ab0b7f576c7ebc863 . "./code.cg2js/cg-lisp-widget.fasl")
(#x0fc19f03b3a88cdbee310263eec66fbb . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x8c0e48bb9568adeac3613deffd0c8a66 . "./code.cg2js/cg-clipboard.fasl")
(#xf41b47a8fc228f90ff5e448d04a6e2c3 . "./code.cg2js/cg-wrap-string.fasl")
(#x9d37a987db5503c0d9e7403d7ed28a55 . "./code.cg2js/cg-os-widget.fasl")
(#x11354b8fbda306b77d42b5cfce21e289 . "./code.cg2js/cg-check-box.fasl")
(#x995a5480b3f39604efbd086af54a7886 . "./code.cg2js/cg-choice-list.fasl")
(#x57363c3cbee448759acb4bbc83a166bc . "./code.cg2js/cg-layout.fasl")
(#xc195d6d3c1d0f0dd34c88981cac60729 . "./code.cg2js/cg-group-box.fasl")
(#x38a5542b3170ada6ce416827698d32fc . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x79a4f759904e19bf9a10479bbe585a32 . "./code.cg2js/cg-radio-button.fasl")
(#x8b1a579db1fb8097e7df82ef70b2050d . "./code.cg2js/cg-tab-control.fasl")
(#x9bd7cdf0261d8c76db975891ca922fa2 . "./code.cg2js/cg.fasl")
(#xb0c3e29e7b0cfdf1b6d3c2585bdc3264 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xdb4c699cf0ed5a2376fde4bc201584b8 . "./code.cg2js/cg-dropping-outline.fasl")
(#x62875d1eaeee0141b9eb3dd720609dd4 . "./code.cg2js/cg-get-pixmap.fasl")
(#xc25a6c73fbd848c8c35b7f525c552900 . "./code.cg2js/cg-grid-widget.fasl")
(#xe414fe99a6937adcd0744d83a26bfbb8 . "./code.cg2js/cg-utility-dialog.fasl")
(#xef7cf83177ef31616ddff00da23bbc53 . "./code.cg2js/cg-static-text.fasl")
(#x768b10c5aef808c5370bc198bacd7059 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x4fd0d23c5477c59bbeed7c21f63b1a08 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x4882394b4028d58734284206a1523f19 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x5ed8340734338e9089f6242f21cdec54 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x460afb33f493ff6cea6636e8d3c04235 . "./code.cg2js/cg-plot-widget.fasl")
(#xa66c59d303170f28f58328bad0e582ea . "./code.cg2js/cg-color-dialog.fasl")
(#x3fbf7b3d0281bca045fc6c7fb546529e . "./code.cg2js/cg-icon-pixmap.fasl")
(#xaf00ca0f2823955331d91d4b3ce9ad23 . "./code.cg2js/cg-icon.fasl")
(#x2d88ff0d5e6697b5e92cd699188b63d0 . "./code.cg2js/cg-dev.fasl")
(#xa2ffbde500fea7235a87f83d2952df03 . "./code.cg2js/cg-choose-printer.fasl")
(#x16e6d67df4813c66a21a7075e61db365 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x62a7bb5c4f43b634e6e5496eb9d544c3 . "./code.cg2js/cg-menu.fasl")
(#xf8102c6cb0257de0dee9698c4036a484 . "./code.cg2js/cg-yes-no-list.fasl")
(#x5a08779eda84adccc4dd3b22baf90e00 . "./code.cg2js/cg-palette.fasl")
(#x9619fceddccf00720d7d0980d8df6d68 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x72f3e45aef2f278457da6be24ff94bc4 . "./code.cg2js/cg-message-dialog.fasl")
(#x846f2618b66b66fedc174ce688c42692 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xfe74d095413695254e18a74af9c55901 . "./code.cg2js/cg-split-bar.fasl")
(#x87926de19f25fac39c43f69d3aaed794 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x2df3c72fb6187186caea4afb1208633b . "./code.cg2js/cg-web-browser.fasl")
(#x56c6282af312eedc5fbfb9ef332bc09c . "./code.cg2js/cg-project-window.fasl")
(#xd8c1c86f32754f4e74c7469bccf256e1 . "./code.cg2js/cg-picture-widget.fasl")
(#xf9418537678ec7e69969a4d7f8278e62 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x04af2c7a8714aa8440b4669d09f846c1 . "./code.cg2js/cg-object-editor.fasl")
(#x399e539e974d4c90c4b704c0a291a644 . "./code.cg2js/cg-paren-matching.fasl")
(#x6190419b88b4e0b81fe07317d21198af . "./code.cg2js/cg-sample-file-menu.fasl")
(#xa285e325d644a0d8567c68b4f9991adf . "./code.cg2js/cg-drawable.fasl")
(#xe7b70955ece44f45ac15465050536a47 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xebaf36b104aa792a39e65d6671e7f751 . "./code.cg2js/cg-html-widget.fasl")
(#x21424c3bb4b046e5c8081d61aa1fb8be . "./code.cg2js/cg-curve.fasl")
(#xa0389a5d6b8912fe1686b1e0f99ef46e . "./code.cg2js/cg-yes-no-string.fasl")
(#x9a21652c165c31417058a1c7e9d611a5 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xbf457c0c1c45890b34d64295a33dda54 . "./code.cg2js/cg-html-dialog.fasl")
(#xb1b4be9f2b264d321a5f8841b9be2e44 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x27390f5e707111e19cc8d83bd5536260 . "./code.cg2js/cg-tray.fasl")
(#xb9b1f2ec98b3cf5644d106b7a0eee313 . "./code.idejs/ide-apropos.fasl")
(#xb94a4b6f2657ba183bfd5c828d6b6daa . "./code.idejs/ide-window-list.fasl")
(#x760a6a1b3116f50b8fe3c1c463689f98 . "./code.idejs/ide-class-browser.fasl")
(#x618dc58446cb97f8053afb5bbe62ec5e . "./code.idejs/ide-base.fasl")
(#x0bdcfe748bf38ced037a5030eadf4325 . "./code.idejs/ide-process-browser.fasl")
(#x6c7a8d4ca7a361ce18f5cc7cacae6eeb . "./code.idejs/ide-acache.fasl")
(#xafb7f401986be4d1932459420b748e6f . "./code.idejs/ide-navigator.fasl")
(#x35a85ae53bae768ad29bc9847e9f1243 . "./code.idejs/ide-trace.fasl")
(#x83c6fd146d4709a81dd622d157ee7818 . "./code.idejs/ide-project.fasl")
(#x314be125e729f6c1faf107a8496e92bd . "./code.idejs/ide-patch-dialog.fasl")
(#x0b7bad5ada1c2df193c41bb6aac398bb . "./code.idejs/ide-grapher.fasl")
(#x45a8ae78ba4c347bfaf1a80ce534fed0 . "./code.idejs/ide-dev.fasl")
(#xde95d32c5379d81b5a2f9f2dfa3efe22 . "./code.idejs/ide-keyshort.fasl")
(#x944207daa57b5d2d1c0208d08d404740 . "./code.idejs/ide-editor.fasl")
(#x3d9ee0588a5c2ca2336f2302d3aa0e50 . "./code.idejs/ide-inspector.fasl")
(#x3995955ab6542e64ea85b25a7008c224 . "./code.idejs/ide.fasl")
(#x173c0a185509fa98a649bd35bf6466b4 . "./code.idejs/ide-clipboard.fasl")
(#x1256359b34ad32362ae233197ad09bdd . "./code.idejs/ide-find-definitions.fasl")
(#x252f12b6a3280b5cd5aed3c335cac8f0 . "./code.idejs/ide-form.fasl")
(#xb8f065ccac288a85371d143da0b6530d . "./code.idejs/ide-intro.fasl")
(#x94c696c0751ac165c305b6b7e14fe17f . "./code.idejs/ide-find-in-files.fasl")
(#xf83d12bf12abb053963ce6fe2f119357 . "./code.idejs/ide-package-list.fasl")
(#x739569ae5fd697a8dc777a0ebf81e689 . "./code.idejs/ide-class-grapher.fasl")
(#x4469068b47e063cb9811630bd05af4c2 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
